import os
import keras
import shutil
import numpy as np
import pandas as pd
import tensorflow as tf
import matplotlib.pyplot as plt

from utility import normalize
from utility import denormalize
from utility import get_datasets

train_ratio = 0.8

tf_epochs = 20
tf_gru_layers = 3

tf_neurons_input_layer = 100;
tf_neurons_per_gru_layer = 8

filename = "sp500.csv"
model_name = "sp500_model"

sess = tf.compat.v1.Session()
sess.run(tf.compat.v1.global_variables_initializer())

tf.compat.v1.keras.backend.set_session(sess)
tf.compat.v1.keras.backend.set_learning_phase(0)
    
dataset = pd.read_csv(filename).values.tolist()

(train_X, train_Y, test_X, test_Y, 
   train_min_vals_X, train_max_vals_X, train_min_vals_Y, train_max_vals_Y,
     test_min_vals_X, test_max_vals_X, test_min_vals_Y, test_max_vals_Y) = get_datasets(dataset, train_ratio)

model = tf.compat.v1.keras.Sequential()

model.add(tf.compat.v1.keras.layers.Dense(
	tf_neurons_input_layer, input_dim=train_X.shape[1]))
model.add(tf.compat.v1.keras.layers.Reshape((int(tf_neurons_input_layer / 10), 
	int(tf_neurons_input_layer / 10)), input_shape=(tf_neurons_input_layer,)))

tf_gru_cells = []
for tf_gru_layer_ii in range(tf_gru_layers):
    tf_gru_cells.append(tf.compat.v1.keras.layers.GRUCell(
	tf_neurons_per_gru_layer, activation='relu'))

model.add(tf.compat.v1.keras.layers.RNN(tf_gru_cells, 
	return_sequences=True, input_shape=(
	  int(tf_neurons_input_layer / 10), int(tf_neurons_input_layer / 10))))

model.add(tf.compat.v1.keras.layers.Flatten())
model.add(tf.compat.v1.keras.layers.Dense(1, input_dim=tf_neurons_per_gru_layer))

Optimizer = tf.compat.v1.keras.optimizers.Adam(lr=0.002, beta_1=0.9, 
	beta_2=0.999, epsilon=None, decay=0.0, amsgrad=False)

model.compile(optimizer=Optimizer, loss='mse')

model.fit(train_X, train_Y, epochs=tf_epochs)

predicted = model.predict(test_X, steps=1)

test_Y    = denormalize(test_Y, test_min_vals_Y, test_max_vals_Y)
predicted = denormalize(predicted, test_min_vals_Y, test_max_vals_Y)

plt.plot(np.array(test_Y), color='blue')
plt.plot(np.array(predicted), color='red')

plt.savefig("sp500_result.jpg", dpi=None, facecolor='w', edgecolor='w',
        orientation='portrait', papertype=None, format=None,
        transparent=False, bbox_inches=None, pad_inches=0.1, metadata=None)

cwd = os.getcwd()
path = os.path.join(cwd, model_name +'/' + '8889')
shutil.rmtree(path, ignore_errors=True, onerror=None)

builder = tf.compat.v1.saved_model.builder.SavedModelBuilder(path)

tensor_info_input = tf.compat.v1.saved_model.utils.build_tensor_info(model.input)
tensor_info_output = tf.compat.v1.saved_model.utils.build_tensor_info(model.outputs[0])

prediction_signature = (
  tf.compat.v1.saved_model.signature_def_utils.build_signature_def(
  inputs={'TICKERS': tensor_info_input},
  outputs={'SP500INDEX': tensor_info_output},
  method_name=tf.compat.v1.saved_model.signature_constants.PREDICT_METHOD_NAME))

builder.add_meta_graph_and_variables(
  sess, [tf.compat.v1.saved_model.tag_constants.SERVING],
  signature_def_map={
  'sp500_prediction':
  prediction_signature
  })
 
builder.save()

print('Tensorflow model has been successfully exported!')